package org.processmining.visualpetrinetanalyzer.algorithms;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.util.AbstractMap.SimpleEntry;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.processmining.visualpetrinetanalyzer.models.Description;
import org.processmining.visualpetrinetanalyzer.models.Event;
import org.processmining.visualpetrinetanalyzer.models.VisualPetrinetModel;

public class Chart {

	public static List<SimpleEntry<Shape, Description>> Draw(Graphics2D ourGraphics, VisualPetrinetModel model, int panelWidth, int panelHeight, int fontSize) {

		List<SimpleEntry<Shape, Description>> rectangles = new ArrayList<SimpleEntry<Shape, Description>>();
		Set<Event> nodes = model.getNodes();
		
		int currentVerticalHeight = (int) (panelHeight * 0.1);
		int currentHorizontalWidth= (int) (panelWidth * 0.1);
		
		int size =  nodes.size();
		
		int verticalNodeHeight = (int) ((panelHeight * 0.9) / size);
		int verticalNodeWidth = (int) (panelWidth * 0.1);
		
		int horizontalNodeWidth= (int) ((panelWidth * 0.9) / size);
		int horizontalNodeHeight= (int) (panelHeight * 0.1);
		
		double highestTime = model.GetHighestValue();
		
		for(Event node : nodes) {
			double eventTime =  model.GetValue(node);
			
			boolean fits = false;
			int fitsSize = fontSize;
			int height = 0;
			int width = 0;
			while(!fits && fitsSize > (1 / 1000000)) {
				Font font = new Font("TimesRoman", Font.PLAIN, fitsSize);
				FontMetrics metrics = ourGraphics.getFontMetrics(font);
				height = metrics.getHeight();
				width = metrics.stringWidth(node.toString());
				
				if(height > verticalNodeHeight) {
					fitsSize *= 0.9;
				} else {
					if (width > verticalNodeWidth) {
						fitsSize *= 0.9;
					} else {
						fits = true;
					}
				}
			}

			Font font = new Font("TimesRoman", Font.PLAIN, fitsSize);
			ourGraphics.setColor(Color.BLACK);
			ourGraphics.setFont(font);							
			//ourGraphics.drawString(node, verticalNodeWidth - width - 5, currentVerticalHeight + ((verticalNodeHeight - height) / 1));

			AffineTransform orig = ourGraphics.getTransform();
			ourGraphics.rotate(Math.PI/2);
			ourGraphics.setColor(Color.BLACK);
			ourGraphics.drawString(node.toString(), (panelHeight - horizontalNodeHeight + 5), -(currentHorizontalWidth + ((horizontalNodeWidth - height) / 2)));
			ourGraphics.setTransform(orig);

			double percentage = 0;
			switch(model.axis) {
				case Normal:
					percentage = eventTime / highestTime;
					break;
				case Log:
					percentage = Math.log(eventTime) / Math.log(highestTime);
					break;
			}

			percentage = Math.min(1, percentage);
			percentage = Math.max(0, percentage);
			
			double red = 255 * percentage;
			double green = 255 * (1 - percentage);

			ourGraphics.setColor(new Color((int)red, (int)green, 0));
			int rectangleHeight = (int)((panelHeight - horizontalNodeHeight) * percentage);
			Rectangle rectangle = new Rectangle(currentHorizontalWidth, panelHeight - horizontalNodeHeight - rectangleHeight, horizontalNodeWidth, rectangleHeight);
			
			ourGraphics.fill(rectangle);
			rectangles.add(new SimpleEntry<Shape, Description>(rectangle, model.getDescription(node)));

			currentVerticalHeight += verticalNodeHeight;
			currentHorizontalWidth += horizontalNodeWidth;
		}
		
		ourGraphics.setColor(Color.BLACK);

		Font font = new Font("TimesRoman", Font.PLAIN, fontSize);
		ourGraphics.setFont(font);							
		
		AffineTransform orig = ourGraphics.getTransform();
		ourGraphics.rotate(Math.PI/2);
		ourGraphics.drawString(model.calculationMethod.toString(), 0, (int)(-panelHeight * 0.1));
		ourGraphics.setTransform(orig);
		
		//X-axis
		ourGraphics.drawLine(verticalNodeWidth, panelHeight - horizontalNodeHeight, panelWidth, panelHeight - horizontalNodeHeight);
		//Y-axis
		ourGraphics.drawLine(verticalNodeWidth, 0, verticalNodeWidth, panelHeight - horizontalNodeHeight);
		
		currentVerticalHeight = (int) (panelHeight * 0.1);
		currentHorizontalWidth= (int) (panelWidth * 0.1);
		for(Event node : nodes) {
			ourGraphics.drawLine(currentHorizontalWidth + horizontalNodeWidth, panelHeight - horizontalNodeHeight, currentHorizontalWidth + horizontalNodeWidth, panelHeight - horizontalNodeHeight - 5);

			currentVerticalHeight += verticalNodeHeight;
			currentHorizontalWidth += horizontalNodeWidth;
		}

		
		return rectangles;
	}
}
