package org.processmining.visualpetrinetanalyzer.algorithms;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.util.AbstractMap.SimpleEntry;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.processmining.visualpetrinetanalyzer.models.Description;
import org.processmining.visualpetrinetanalyzer.models.Event;
import org.processmining.visualpetrinetanalyzer.models.VisualPetrinetModel;

public class Matrix {

	public static List<SimpleEntry<Shape, Description>> Draw(Graphics2D ourGraphics, VisualPetrinetModel model, int panelWidth, int panelHeight, int fontSize) {

		Set<Event> nodes = model.getNodes();
		
		int currentVerticalHeight = (int) (panelHeight * 0.1);
		int currentHorizontalWidth= (int) (panelWidth * 0.1);
		
		int size =  nodes.size();
		
		int verticalNodeHeight = (int) ((panelHeight * 0.9) / size);
		int verticalNodeWidth = (int) (panelWidth * 0.1);
		
		int horizontalNodeWidth= (int) ((panelWidth * 0.9) / size);
		int horizontalNodeHeight= (int) (panelHeight * 0.1);
		
		for(Event node : nodes) {
			boolean fits = false;
			int fitsSize = fontSize;
			int height = 0;
			int width = 0;
			while(!fits && fitsSize > (1 / 1000000)) {
				Font font = new Font("TimesRoman", Font.PLAIN, fitsSize);
				FontMetrics metrics = ourGraphics.getFontMetrics(font);
				height = metrics.getHeight();
				width = metrics.stringWidth(node.toString());
				
				if(height > verticalNodeHeight) {
					fitsSize *= 0.9;
				} else {
					if (width > verticalNodeWidth) {
						fitsSize *= 0.9;
					} else {
						fits = true;
					}
				}
			}

			Font font = new Font("TimesRoman", Font.PLAIN, fitsSize);
			ourGraphics.setColor(Color.BLACK);
			ourGraphics.setFont(font);							
			ourGraphics.drawString(node.toString(), verticalNodeWidth - width - 5, currentVerticalHeight + ((verticalNodeHeight - height) / 1));

			AffineTransform orig = ourGraphics.getTransform();
			ourGraphics.rotate(Math.PI/2);
			ourGraphics.setColor(Color.BLACK);
			ourGraphics.drawString(node.toString(), horizontalNodeHeight - width - 5, -(currentHorizontalWidth + ((horizontalNodeWidth - height) / 2)));
			ourGraphics.setTransform(orig);

			currentVerticalHeight += verticalNodeHeight;
			currentHorizontalWidth += horizontalNodeWidth;
		}
		
		int xOffset = (int) (panelWidth * 0.1);
		int yOffset = (int) (panelHeight * 0.1);
		int width = panelWidth - xOffset;
		int height = panelHeight - yOffset;

		Font font = new Font("TimesRoman", Font.PLAIN, fontSize);
		AffineTransform orig = ourGraphics.getTransform();		

		ourGraphics.setColor(Color.BLACK);
		ourGraphics.setFont(font);						
		ourGraphics.rotate(Math.PI/2);
		ourGraphics.drawString("Sources ->", 0, 0);
		ourGraphics.setTransform(orig);
		ourGraphics.drawString("Targets ->", 0, 0);
		
		return createRaster(ourGraphics, model, width, height, xOffset, yOffset);
	}
	
	private static List<SimpleEntry<Shape, Description>> createRaster(Graphics2D ourGraphics, VisualPetrinetModel model, int width, int height, int xOffset, int yOffset) {

		List<SimpleEntry<Shape, Description>> rectangles = new ArrayList<SimpleEntry<Shape, Description>>();
		Set<Event> nodes = model.getNodes();

		int size =  nodes.size();
		int cellWidth = width / size;
		int cellHeight= height / size;

		int stepX = xOffset;
		int stepY = yOffset;

		double highestTime = model.GetHighestValuePerSource();
		
		for(Event target : nodes) {	
			for(Event source : nodes) {			
				if(model.targetHasSource(target, source)) {
					double eventTime = model.GetValue(target, source);
					
					double percentage = 0;
					switch(model.axis) {
						case Normal:
							percentage = eventTime / highestTime;
							break;
						case Log:
							percentage = Math.log(eventTime) / Math.log(highestTime);
							break;
					}
					
					percentage = Math.min(1, percentage);
					percentage = Math.max(0, percentage);
					
					double red = 255 * percentage;
					double green = 255 * (1 - percentage);
					
					ourGraphics.setColor(new Color((int)red, (int)green, 0));
					Rectangle rectangle = new Rectangle(stepX, stepY, cellWidth, cellHeight);
					ourGraphics.fill(rectangle);
					rectangles.add(new SimpleEntry<Shape, Description>(rectangle, model.getDescription(target, source)));
				}
				
				ourGraphics.setColor(Color.BLACK);				
				ourGraphics.drawRect(stepX, stepY, cellWidth, cellHeight);

				stepY += cellHeight;
			}
			stepY = yOffset;
			stepX += cellWidth;
		}
		
		return rectangles;
	}
}
