package org.processmining.visualpetrinetanalyzer.algorithms;

import java.util.Date;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.visualpetrinetanalyzer.models.Event;
import org.processmining.visualpetrinetanalyzer.models.VisualPetrinetModel;
import org.processmining.visualpetrinetanalyzer.parameters.VisualPetrinetModelParameters;

public class VisualPetrinetAnalyzerAlgorithm {
	
	protected VisualPetrinetModel mine(PluginContext context, XLog log, Petrinet petrinet, VisualPetrinetModelParameters parameters){
		
		VisualPetrinetModel model = new VisualPetrinetModel(petrinet, parameters);
		
		context.getProgress().setMaximum(log.size());
				
		XLogInfo info = XLogInfoFactory.createLogInfo(log);

		for (int i = 0; i < log.size(); i++) {
			for (int t = 1; t < log.get(i).size(); t++) {
				XEvent targetEvent = log.get(i).get(t);
				
				Event target = extractEvent(targetEvent);
				
				model.IncreaseFrequency(target);
				
				for (int s = t - 1; s >= 0; s--) {
					//Walk back in the log
					XEvent sourceEvent = log.get(i).get(s);
					Event source = extractEvent(sourceEvent);
					
					if(model.targetHasSource(target, source)) {
						Date endDate = XTimeExtension.instance().extractTimestamp(targetEvent);
						Date startDate = XTimeExtension.instance().extractTimestamp(sourceEvent);
						long diff = (endDate != null && startDate != null ? endDate.getTime() - startDate.getTime() : 1);
						model.addTimestamp(target, source, diff);	
						//If the event that cause the execution of the target is found -> go to next event in trace
						break;
					}
				}
			}
			
		    context.getProgress().inc();			
		}
		
		return model;
	}
	
	private Event extractEvent(XEvent xEvent) {
		String eventName = XConceptExtension.instance().extractName(xEvent);
		if (eventName == null) {
			eventName = XConceptExtension.KEY_NAME + " attribute is missing";
		}
		String transition = XLifecycleExtension.instance().extractTransition(xEvent);
		if (transition == null) {
			transition = XLifecycleExtension.KEY_TRANSITION + " attribute is missing";
		}
		return new Event(eventName, transition);
	}
	
}
