package org.processmining.visualpetrinetanalyzer.models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

public class TimeRecords {

	private Event Name;
	private Map<Event, List<Double>> SourceTimes;
	private int Frequency;
	
	public TimeRecords(Event name) {
		Name = name;
		SourceTimes = new HashMap<Event, List<Double>>();
		Frequency = 0;
	}
	
	public void AddSource(Event source) {
		SourceTimes.put(source, new ArrayList<Double>());
	}
	
	public void AddSourceTime(Event source, double time) {
		if(SourceTimes.containsKey(source)) {
			SourceTimes.get(source).add(time);
		}
	}

	public boolean HasSource(Event source) {
		return SourceTimes.containsKey(source);
	}
	
	public Set<Event> GetSource() {
		return SourceTimes.keySet();
	}
	
	public void IncreaseFrequency() {
		Frequency++;
	}
	
	public int GetFrequency() {
		return Frequency;
	}
	
	public int GetPredFrequency() {
		int predFrequency = 0;
		for(List<Double> values : SourceTimes.values()) {
			predFrequency += values.size();
		}
		return predFrequency;
	}
	
	public int GetPredFrequency(Event source) {
		int predFrequency = 0;
		if(SourceTimes.containsKey(source)) {
			predFrequency = SourceTimes.get(source).size();
		}
		return predFrequency;
	}
	
	public Description getDescription(Event source) {
		Description description = new Description(Name.toString(), source.toString());
		if(SourceTimes.containsKey(source)) {
			description.AddItem(String.format("Frequency: %d\n", GetFrequency()));
			description.AddItem(String.format("Predecessors: %d\n", GetPredFrequency(source)));
			description.AddItem(String.format("Sum: %s\n", getTimeString(getSum(source))));
			description.AddItem(String.format("Average: %s\n", getTimeString(getAverage(source))));
			description.AddItem(String.format("Max: %s\n", getTimeString(getMax(source))));
			description.AddItem(String.format("Min: %s\n", getTimeString(getMin(source))));
			description.AddItem(String.format("StdDev: %s", getTimeString(getStdDev(source))));
		}
		return description;
	}
	
	public Description getDescription() {
		Description description = new Description(Name.toString());
		description.AddItem(String.format("Frequency: %d\n", GetFrequency()));
		description.AddItem(String.format("Predecessors: %d\n", GetPredFrequency()));
		description.AddItem(String.format("Sum: %s\n", getTimeString(getSum())));
		description.AddItem(String.format("Average: %s\n", getTimeString(getAverage())));
		description.AddItem(String.format("Max: %s\n", getTimeString(getMax())));
		description.AddItem(String.format("Min: %s\n", getTimeString(getMin())));
		description.AddItem(String.format("StdDev: %s", getTimeString(getStdDev())));
		return description;
	}
	
	public double getSum(Event source) {
		double sum = 0;
		if(SourceTimes.containsKey(source)) {
			for(double time : SourceTimes.get(source)) {
				sum += time;
			}
		}
		return sum;
	}
	
	public double getSum() {
		long sum = 0;
		for(Event source : SourceTimes.keySet()) {
			sum += getSum(source);
		}
		return sum;
	}
	
	public double getAverage(Event source) {
		double sum = 0;
		double count = 0;
		if(SourceTimes.containsKey(source)) {
			for(double time : SourceTimes.get(source)) {
				sum += time;
				count++;
			}
		}
		return (sum / count);
	}
	
	public double getAverage() {
		double sum = 0;
		double count = 0;
		for(List<Double> values : SourceTimes.values()) {
			for(double time : values) {
				sum += time;
				count++;
			}
		}
		return (sum / count);		
	}
	
	public double getMax(Event source) {
		double max = 0;
		if(SourceTimes.containsKey(source)) {
			for(double time : SourceTimes.get(source)) {
				if(time > max) {
					max = time;
				}
			}
		}
		return max;
	}
	
	public double getMax() {
		double max = 0;
		for(Event source : SourceTimes.keySet()) {
			double sourceMax = getMax(source);
			if(sourceMax > max) {
				max = sourceMax;
			}
		}
		return max;
	}

	public double getMin(Event source) {
		double min = 0;
		if(SourceTimes.containsKey(source)) {
			for(double time : SourceTimes.get(source)) {
				if(time < min || min == 0) {
					min = time;
				}
			}
		}
		return min;		
	}
	
	public double getMin() {
		double min = 0;
		for(Event source : SourceTimes.keySet()) {
			double sourceMin = getMin(source);
			if(sourceMin < min || min == 0) {
				min = sourceMin;
			}
		}
		return min;		
	}
	
	public double getStdDev(Event source) {
		double mean = getAverage(source);
		double count = 0;
        double temp = 0;
		if(SourceTimes.containsKey(source)) {
			for(double time : SourceTimes.get(source)) {
				count++;
	            temp += (mean - time) * (mean - time);
			}
		}
        return Math.sqrt(temp / count);
	}
	
	public double getStdDev() {
		double mean = getAverage();
		long count = 0;
        double temp = 0;
		for(List<Double> values : SourceTimes.values()) {
			for(double time : values) {
				count++;
	            temp += (mean - time) * (mean - time);
			}
		}
        return  Math.sqrt(temp / count);
	}
	
	private static String getTimeString(double millis) {
		long time = (long)millis;
		long days = TimeUnit.MILLISECONDS.toDays(time);
		time -= TimeUnit.DAYS.toMillis(days);
        long hours = TimeUnit.MILLISECONDS.toHours(time);
        time -= TimeUnit.HOURS.toMillis(hours);
        long minutes = TimeUnit.MILLISECONDS.toMinutes(time);
        time -= TimeUnit.MINUTES.toMillis(minutes);
        long seconds = TimeUnit.MILLISECONDS.toSeconds(time);
		
		return String.format("%d day, %d hour, %d min, %d sec", days, hours, minutes, seconds);			
	}	
}
