package org.processmining.visualpetrinetanalyzer.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.visualpetrinetanalyzer.algorithms.VisualPetrinetAnalyzerAlgorithm;
import org.processmining.visualpetrinetanalyzer.dialogs.VisualPetrinetMiningDialog;
import org.processmining.visualpetrinetanalyzer.models.VisualPetrinetModel;
import org.processmining.visualpetrinetanalyzer.parameters.VisualPetrinetModelParameters;

@Plugin(name = "Visual Petrinet Analyzer", returnLabels = { "Petrinet Visualization" }, returnTypes = { VisualPetrinetModel.class }, parameterLabels = { "Log", "Petrinet" }, userAccessible = true)
public class VisualPetrinetAnalyzerPlugin extends VisualPetrinetAnalyzerAlgorithm {
	
	@UITopiaVariant(author = "J. Reijrink", email = "j.reijrink@student.tue.nl", affiliation = "TU Eindhoven")
	@PluginVariant(variantLabel = "Visual Petrinet Analyzer, default", requiredParameterLabels = { 0, 1 })
	public VisualPetrinetModel mineDefault(UIPluginContext context, XLog log, Petrinet petrinet){
		
		VisualPetrinetModelParameters parameters = new VisualPetrinetModelParameters();
		VisualPetrinetMiningDialog dialog = new VisualPetrinetMiningDialog(parameters);
		InteractionResult result = context.showWizard("Visual Petrinet Analyzer", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		
		return mine(context, log, petrinet, parameters);
	}
		
}
